@@ -306,7 +306,6 @@ pango_default_break (const gchar   *text,
       /* Emoji extended pictographics */
       gboolean is_Extended_Pictographic;
 
-
       wc = next_wc;
       break_type = next_break_type;
 
@@ -362,12 +361,15 @@ pango_default_break (const gchar   *text,
         case G_UNICODE_PARAGRAPH_SEPARATOR:
           attrs[i].is_white = TRUE;
           break;
-        default:
+        case G_UNICODE_CONTROL:
           if (wc == '\t' || wc == '\n' || wc == '\r' || wc == '\f')
             attrs[i].is_white = TRUE;
           else
             attrs[i].is_white = FALSE;
           break;
+        default:
+          attrs[i].is_white = FALSE;
+          break;
         }
 
       /* Just few spaces have variable width. So explicitly mark them.
@@ -502,9 +504,7 @@ pango_default_break (const gchar   *text,
 	else if (GB_type == GB_InHangulSyllable)
 	  is_grapheme_boundary = FALSE; /* Rules GB6, GB7, GB8 */
 	else if (GB_type == GB_Extend)
-          {
-	    is_grapheme_boundary = FALSE; /* Rule GB9 */
-          }
+	  is_grapheme_boundary = FALSE; /* Rule GB9 */
         else if (GB_type == GB_ZWJ)
 	  is_grapheme_boundary = FALSE; /* Rule GB9 */
 	else if (GB_type == GB_SpacingMark)
@@ -641,14 +641,12 @@ pango_default_break (const gchar   *text,
 		  if (wc >= 0x24B6 && wc <= 0x24E9) /* Other_Alphabetic */
 		    goto Alphabetic;
 
-		  if (G_UNLIKELY(wc >=0x1F1E6 && wc <=0x1F1FF))
+		  if (G_UNLIKELY(wc >= 0x1F1E6 && wc <= 0x1F1FF))
 		    {
-			  if (prev_WB_type == WB_RI_Odd)
-			   WB_type = WB_RI_Even;
-			  else if (prev_WB_type == WB_RI_Even)
-			   WB_type = WB_RI_Odd;
-			  else
-			   WB_type = WB_RI_Odd;
+                      if (prev_WB_type == WB_RI_Odd)
+                        WB_type = WB_RI_Even;
+                      else
+                        WB_type = WB_RI_Odd;
 		    }
 
 		  break;
@@ -933,7 +931,20 @@ pango_default_break (const gchar   *text,
 		      prev_prev_SB_type == SB_ATerm_Close_Sp) &&
 		     IS_OTHER_TERM(prev_SB_type) &&
 		     SB_type == SB_Lower)
-	      attrs[prev_SB_i].is_sentence_boundary = FALSE;
+              {
+	        attrs[prev_SB_i].is_sentence_boundary = FALSE;
+	        attrs[prev_SB_i].is_sentence_start = FALSE;
+	        attrs[prev_SB_i].is_sentence_end = FALSE;
+                last_sentence_start = -1;
+                for (int j = prev_SB_i - 1; j >= 0; j--)
+                  {
+                    if (attrs[j].is_sentence_boundary)
+                      {
+                        last_sentence_start = j;
+                        break;
+                      }
+                  }
+              }
 	    else if ((prev_SB_type == SB_ATerm ||
 		      prev_SB_type == SB_ATerm_Close_Sp ||
 		      prev_SB_type == SB_STerm ||
@@ -1538,8 +1549,9 @@ pango_default_break (const gchar   *text,
 	/* meets space character, move sentence start */
 	if (last_sentence_start != -1 &&
 	    last_sentence_start == i - 1 &&
-	    attrs[i - 1].is_white)
+	    attrs[i - 1].is_white) {
 	    last_sentence_start++;
+          }
 
       }
 
@@ -1563,7 +1575,6 @@ pango_default_break (const gchar   *text,
 
   attrs[i].is_line_break = TRUE;  /* Rule LB3 */
   attrs[0].is_line_break = FALSE; /* Rule LB2 */
-
 }
 
 static gboolean
