@@ -19,18 +19,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:opentype
- * @short_description:Obtaining information from OpenType tables
- * @title:OpenType Font Handling
- * @stability:Unstable
- *
- * Functions and macros in this section are used to implement
- * the OpenType Layout features and algorithms.
- *
- * They have been superseded by the harfbuzz library, and should
- * not be used anymore.
- */
 #include "config.h"
 
 #include "pango-ot-private.h"
@@ -76,7 +64,7 @@ pango_ot_info_finalizer (void *object)
 
 /**
  * pango_ot_info_get:
- * @face: a <type>FT_Face</type>.
+ * @face: a FT_Face
  *
  * Returns the #PangoOTInfo structure for the given FreeType font face.
  *
@@ -184,7 +172,7 @@ pango_ot_info_find_language (PangoOTInfo      *info,
 			     guint            *required_feature_index)
 {
   gboolean ret;
-  unsigned l_index;
+  guint l_index;
   hb_tag_t tt = get_hb_table_type (table_type);
 
   ret = hb_ot_layout_script_select_language (info->hb_face,
@@ -192,8 +180,9 @@ pango_ot_info_find_language (PangoOTInfo      *info,
                                              script_index,
                                              1,
                                              &language_tag,
-                                             language_index);
-  if (language_index) *language_index = l_index;
+                                             &l_index);
+  if (language_index)
+    *language_index = l_index;
 
   hb_ot_layout_language_get_required_feature_index (info->hb_face, tt,
 						    script_index,
