@@ -53,15 +53,6 @@
  * of the copyright holder.
  */
 
-/**
- * SECTION:scripts
- * @short_description:Identifying writing systems and languages
- * @title:Scripts and Languages
- *
- * The functions in this section are used to identify the writing
- * system, or <firstterm>script</firstterm> of individual characters
- * and of ranges within a larger text string.
- */
 #include "config.h"
 #include <stdlib.h>
 #include <string.h>
@@ -73,17 +64,18 @@
  * pango_script_for_unichar:
  * @ch: a Unicode character
  *
- * Looks up the script for a particular character (as defined by
- * Unicode Standard Annex \#24). No check is made for @ch being a
- * valid Unicode character; if you pass in invalid character, the
- * result is undefined.
+ * Looks up the script for a particular character.
+ *
+ * The script of a character is defined by Unicode Standard Annex \#24.
+ * No check is made for @ch being a valid Unicode character; if you pass
+ * in invalid character, the result is undefined.
  *
  * Note that while the return type of this function is declared
- * as PangoScript, as of Pango 1.18, this function simply returns
+ * as `PangoScript`, as of Pango 1.18, this function simply returns
  * the return value of g_unichar_get_script(). Callers must be
  * prepared to handle unknown values.
  *
- * Return value: the #PangoScript for the character.
+ * Return value: the `PangoScript` for the character.
  *
  * Since: 1.4
  * Deprecated: 1.44. Use g_unichar_get_script()
@@ -96,6 +88,13 @@ pango_script_for_unichar (gunichar ch)
 
 /**********************************************************************/
 
+static PangoScriptIter *pango_script_iter_copy (PangoScriptIter *iter);
+
+G_DEFINE_BOXED_TYPE (PangoScriptIter,
+                     pango_script_iter,
+                     pango_script_iter_copy,
+                     pango_script_iter_free)
+
 PangoScriptIter *
 _pango_script_iter_init (PangoScriptIter *iter,
 	                 const char      *text,
@@ -142,6 +141,12 @@ pango_script_iter_new (const char *text,
   return _pango_script_iter_init (g_slice_new (PangoScriptIter), text, length);
 }
 
+static PangoScriptIter *
+pango_script_iter_copy (PangoScriptIter *iter)
+{
+  return g_slice_dup (PangoScriptIter, iter);
+}
+
 void
 _pango_script_iter_fini (PangoScriptIter *iter)
 {
