@@ -106,7 +106,7 @@
  */
 static GMutex fc_init_mutex;
 static GCond fc_init_cond;
-static gboolean fc_initialized;
+static int fc_initialized = 0;
 
 
 typedef struct _PangoFcFontFaceData PangoFcFontFaceData;
@@ -820,11 +820,8 @@ thread_data_free (gpointer data)
   g_object_unref (fontmap);
 }
 
-static void
-match_in_thread (GTask        *task,
-                 gpointer      source_object,
-                 gpointer      task_data,
-                 GCancellable *cancellable)
+static gpointer
+match_in_thread (gpointer task_data)
 {
   ThreadData *td = task_data;
   FcResult result;
@@ -842,13 +839,14 @@ match_in_thread (GTask        *task,
   td->patterns->match = match;
   g_cond_signal (&td->patterns->cond);
   g_mutex_unlock (&td->patterns->mutex);
+
+  thread_data_free (td);
+
+  return NULL;
 }
 
-static void
-sort_in_thread (GTask        *task,
-                gpointer      source_object,
-                gpointer      task_data,
-                GCancellable *cancellable)
+static gpointer
+sort_in_thread (gpointer task_data)
 {
   ThreadData *td = task_data;
   FcResult result;
@@ -868,13 +866,17 @@ sort_in_thread (GTask        *task,
   td->patterns->fontset = fontset;
   g_cond_signal (&td->patterns->cond);
   g_mutex_unlock (&td->patterns->mutex);
+
+  thread_data_free (td);
+
+  return NULL;
 }
 
 static PangoFcPatterns *
 pango_fc_patterns_new (FcPattern *pat, PangoFcFontMap *fontmap)
 {
   PangoFcPatterns *pats;
-  GTask *task;
+  GThread *thread;
 
   pat = uniquify_pattern (fontmap, pat);
   pats = g_hash_table_lookup (fontmap->priv->patterns_hash, pat);
@@ -892,17 +894,11 @@ pango_fc_patterns_new (FcPattern *pat, PangoFcFontMap *fontmap)
   g_mutex_init (&pats->mutex);
   g_cond_init (&pats->cond);
 
-  task = g_task_new (NULL, NULL, NULL, NULL);
-  g_task_set_name (task, "[pango] FcFontSetMatch");
-  g_task_set_task_data (task, thread_data_new (pats), thread_data_free);
-  g_task_run_in_thread (task, match_in_thread);
-  g_object_unref (task);
+  thread = g_thread_new ("[pango] FcFontSetMatch", match_in_thread, thread_data_new (pats));
+  g_thread_unref (thread);
 
-  task = g_task_new (NULL, NULL, NULL, NULL);
-  g_task_set_name (task, "[pango] FcFontSetSort");
-  g_task_set_task_data (task, thread_data_new (pats), thread_data_free);
-  g_task_run_in_thread (task, sort_in_thread);
-  g_object_unref (task);
+  thread = g_thread_new ("[pango] FcFontSetSort", sort_in_thread, thread_data_new (pats));
+  g_thread_unref (thread);
 
   g_hash_table_insert (fontmap->priv->patterns_hash,
                        pats->pattern, pats);
@@ -1344,11 +1340,8 @@ G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PangoFcFontMap, pango_fc_font_map, PANGO_TYPE_
                                   G_ADD_PRIVATE (PangoFcFontMap)
                                   G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, pango_fc_font_map_list_model_init))
 
-static void
-init_in_thread (GTask        *task,
-                gpointer      source_object,
-                gpointer      task_data,
-                GCancellable *cancellable)
+static gpointer
+init_in_thread (gpointer task_data)
 {
   gint64 before = PANGO_TRACE_CURRENT_TIME;
 
@@ -1357,8 +1350,27 @@ init_in_thread (GTask        *task,
   pango_trace_mark (before, "FcInit", NULL);
 
   g_mutex_lock (&fc_init_mutex);
-  fc_initialized = TRUE;
-  g_cond_signal (&fc_init_cond);
+  fc_initialized = 2;
+  g_cond_broadcast (&fc_init_cond);
+  g_mutex_unlock (&fc_init_mutex);
+
+  return NULL;
+}
+
+static void
+start_init_in_thread (PangoFcFontMap *fcfontmap)
+{
+  g_mutex_lock (&fc_init_mutex);
+
+  if (fc_initialized == 0)
+    {
+      GThread *thread;
+
+      fc_initialized = 1;
+      thread = g_thread_new ("[pango] FcInit", init_in_thread, NULL);
+      g_thread_unref (thread);
+    }
+
   g_mutex_unlock (&fc_init_mutex);
 }
 
@@ -1369,7 +1381,7 @@ wait_for_fc_init (void)
   gboolean waited = FALSE;
 
   g_mutex_lock (&fc_init_mutex);
-  while (!fc_initialized)
+  while (fc_initialized < 2)
     {
       waited = TRUE;
       g_cond_wait (&fc_init_cond, &fc_init_mutex);
@@ -1411,15 +1423,7 @@ pango_fc_font_map_init (PangoFcFontMap *fcfontmap)
 						     NULL);
   priv->dpi = -1;
 
-  if (!fc_initialized)
-    {
-      GTask *task;
-
-      task = g_task_new (fcfontmap, NULL, NULL, NULL);
-      g_task_set_name (task, "[pango] FcInit");
-      g_task_run_in_thread (task, init_in_thread);
-      g_object_unref (task);
-    }
+  start_init_in_thread (fcfontmap);
 }
 
 static void
@@ -2321,7 +2325,7 @@ pango_fc_font_map_set_config (PangoFcFontMap *fcfontmap,
     FcConfigDestroy (oldconfig);
 
   /* No need to wait anymore */
-  fc_initialized = TRUE;
+  fc_initialized = 2;
 }
 
 /**
@@ -2893,7 +2897,8 @@ pango_fc_face_list_sizes (PangoFontFace  *face,
   FcObjectSet *objectset;
   FcFontSet *fonts;
 
-  *sizes = NULL;
+  if (sizes)
+    *sizes = NULL;
   *n_sizes = 0;
   if (G_UNLIKELY (!fcface->family || !fcface->family->fontmap))
     return;
