@@ -13,7 +13,7 @@ from pygments.lexers import get_lexer_by_name
 from pygments.formatters import HtmlFormatter
 from typogrify.filters import typogrify
 
-from . import gir, log, mdext
+from . import gir, log, mdext, porter
 
 
 # The beginning of a gtk-doc code block:
@@ -578,7 +578,7 @@ class LinkGenerator:
             return f"<a href=\"{link}\">{text}</a>"
 
 
-def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plain=False):
+def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plain=False, max_length=10):
     processed_text = []
 
     code_block_text = []
@@ -642,7 +642,15 @@ def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plai
                 processed_text.append("".join(new_line))
 
     if plain:
-        return "\n".join(processed_text)
+        text = text.replace('\n', ' ')
+        text = re.sub(r'<[^<]+?>', '', text)
+        if max_length > 0:
+            words = text.split(' ')
+            if len(words) > max_length:
+                words = words[:max_length - 1]
+                words.append('...')
+                text = ' '.join(words)
+        return text
 
     if md is None:
         md_ext = extensions.copy()
@@ -656,7 +664,13 @@ def preprocess_docs(text, namespace, summary=False, md=None, extensions=[], plai
     return Markup(typogrify(text, ignore_tags=['h1', 'h2', 'h3', 'h4']))
 
 
-def preprocess_index(text):
+def stem(word, stemmer=None):
+    if stemmer is None:
+        stemmer = porter.PorterStemmer()
+    return stemmer.stem(word, 0, len(word) - 1)
+
+
+def index_description(text, stemmer=None):
     processed_text = []
 
     inside_code_block = False
@@ -670,39 +684,29 @@ def preprocess_index(text):
             continue
 
         if not inside_code_block:
-            new_line = []
-            idx = 0
-            for m in LINK_RE.finditer(line, idx):
-                start = m.start()
-                end = m.end()
-                left_pad = line[idx:start]
-                replacement = re.sub(LINK_RE, '', line[start:end])
-                new_line.append(left_pad)
-                new_line.append(replacement)
-                idx = end
-            new_line.append(line[idx:])
-
-            if len(new_line) == 0:
-                processed_text.append(line)
-            else:
-                processed_text.append("".join(new_line))
+            processed_text.append(line)
 
     data = " ".join(processed_text)
     terms = set()
     for chunk in data.split(" "):
+        chunk = chunk.lower()
         if chunk in ["\n", "\r", "\r\n"]:
             continue
+        # Skip gtk-doc sygils
+        if chunk.startswith('%') or chunk.startswith('#') or chunk.startswith('@') or chunk.endswith('()'):
+            continue
+        # Skip gi-docgen links
+        if chunk.startswith('[') and chunk.endswith(']') and '@' in chunk:
+            continue
+        # Skip images
+        if chunk.startswith('!['):
+            continue
+        if chunk in EN_STOPWORDS:
+            continue
         chunk = re.sub(r"`(\w+)`", r"\g<1>", chunk)
         chunk = re.sub(r"[,\.:;`]$", '', chunk)
-        if chunk.startswith('%') or chunk.startswith('#') or chunk.startswith('@'):
-            continue
-        if chunk.startswith('!['):
-            continue
         chunk = re.sub(r"[\(\)]+", '', chunk)
-        term = chunk.lower()
-        if term in EN_STOPWORDS:
-            continue
-        terms.add(term)
+        terms.add(stem(chunk, stemmer))
     return terms
 
 
@@ -710,7 +714,7 @@ def canonicalize(symbol):
     return symbol.replace('-', '_')
 
 
-def index_identifier(symbol):
+def index_identifier(symbol, stemmer=None):
     """Chunks an identifier (e.g. EventControllerClik) into terms useful for indexing."""
     symbol = re.sub(CAMEL_CASE_START_RE, r"\g<1>_\g<2>", symbol)
     symbol = re.sub(CAMEL_CASE_CHUNK_RE, r"\g<1>_\g<2>", symbol)
@@ -720,17 +724,17 @@ def index_identifier(symbol):
     for chunk in symbol.split('_'):
         if chunk in EN_STOPWORDS:
             continue
-        terms.add(chunk)
+        terms.add(stem(chunk, stemmer))
     return terms
 
 
-def index_symbol(symbol):
+def index_symbol(symbol, stemmer=None):
     """Chunks a symbol (e.g. set_layout_manager) into terms useful for indexing."""
     terms = set()
     for chunk in canonicalize(symbol).split('_'):
         if chunk in EN_STOPWORDS:
             continue
-        terms.add(chunk)
+        terms.add(stem(chunk, stemmer))
     return terms
 
 
@@ -802,3 +806,21 @@ def find_program(bin_name, path=None):
                 return full_path
 
     return None
+
+
+def default_search_paths():
+    if not sys.platform == 'win32':
+        xdg_data_dirs = os.environ.get("XDG_DATA_DIRS", "/usr/share:/usr/local/share").split(":")
+        xdg_data_home = os.environ.get("XDG_DATA_HOME", os.path.expanduser("~/.local/share"))
+    else:
+        xdg_data_dirs = None
+        xdg_data_home = None
+
+    paths = []
+    paths.append(os.getcwd())
+    if xdg_data_home is not None:
+        paths.append(os.path.join(xdg_data_home, "gir-1.0"))
+    if xdg_data_dirs is not None:
+        paths.extend([os.path.join(x, "gir-1.0") for x in xdg_data_dirs])
+
+    return paths
