@@ -47,15 +47,12 @@ test_file (const gchar *filename, GString *string)
   gboolean ret;
   char *str;
   int start, end;
+  gunichar accel = 0;
 
-  if (!g_file_get_contents (filename, &contents, &length, &error))
-    {
-      fprintf (stderr, "%s\n", error->message);
-      g_error_free (error);
-      return;
-    }
+  g_file_get_contents (filename, &contents, &length, &error);
+  g_assert_no_error (error);
 
-  ret = pango_parse_markup (contents, length, 0, &attrs, &text, NULL, &error);
+  ret = pango_parse_markup (contents, length, '_', &attrs, &text, &accel, &error);
   g_free (contents);
 
   if (ret)
@@ -78,6 +75,12 @@ test_file (const gchar *filename, GString *string)
       pango_attr_list_unref (attrs);
       pango_font_description_free (desc);
       g_free (text);
+
+      if (accel)
+        {
+          g_string_append (string, "\n\n---\n\n");
+          g_string_append_unichar (string, accel);
+        }
     }
   else
     {
@@ -143,10 +146,8 @@ main (int argc, char *argv[])
   g_setenv ("LC_ALL", "C", TRUE);
   setlocale (LC_ALL, "");
 
-  g_test_init (&argc, &argv, NULL);
-
   /* allow to easily generate expected output for new test cases */
-  if (argc > 1)
+  if (argc > 1 && argv[1][0] != '-')
     {
       GString *string;
 
@@ -157,6 +158,8 @@ main (int argc, char *argv[])
       return 0;
     }
 
+  g_test_init (&argc, &argv, NULL);
+
   path = g_test_build_filename (G_TEST_DIST, "markups", NULL);
   dir = g_dir_open (path, 0, &error);
   g_free (path);
